<?php

/********************************************************************
 *   Autori:             Marco Colombo (marco@openit.it)
 *                       Valentina Portolan (valentina@openit.it)
 *                       Giuliano Natali Diaolin (diaolin@openit.it)
 *  Copyright:           Open It S.r.l.
 *                       Viale Dante, 78
 *                       38057 Pergine Valsugana (TN) ITALY
 *                       Tel: +39 0461 534800 Fax: +39 0461 538443
 *********************************************************************/

/**
 * abook_local_file.php
 *
 * Copyright (c) 1999-2003 The SquirrelMail Project Team
 * Licensed under the GNU GPL. For full terms see the file COPYING.
 *
 * Backend for addressbook as a pipe separated file
 *
 * An array with the following elements must be passed to
 * the class constructor (elements marked ? are optional):
 *
 *    filename  => path to addressbook file
 *  ? create    => if true: file is created if it does not exist.
 *  ? umask     => umask set before opening file.
 *
 * NOTE. This class should not be used directly. Use the
 *       "AddressBook" class instead.
 *
 * $Id: abook_local_file.php,v 1.7 2004/06/02 07:40:39 marco Exp $
 */

if (!defined('SM_PATH')) { define('SM_PATH','../'); }

class abook_local_file extends addressbook_backend {
    var $btype = 'local';
    var $bname = 'local_file';

    var $filename   = '';
    var $filehandle = 0;
    var $create     = false;
    var $umask;

    /* ========================== Private ======================= */

    /* Constructor */
    function abook_local_file($param) {
        $this->sname = _("Personal address book");
        $this->umask = Umask();

        if(is_array($param)) {
            if(empty($param['filename'])) {
                return $this->set_error('Invalid parameters');
            }
            if(!is_string($param['filename'])) {
                return $this->set_error($param['filename'] . ': '.
                     _("Not a file name"));
            }

            $this->filename = $param['filename'];

            if($param['create']) {
                $this->create = true;
            }
            if(isset($param['umask'])) {
                $this->umask = $param['umask'];
            }
            if(!empty($param['name'])) {
                $this->sname = $param['name'];
            }
           
            $this->open(true);
        } else {
            $this->set_error('Invalid argument to constructor');
        }
    }

    /* Open the addressbook file and store the file pointer.
     * Use $file as the file to open, or the class' own 
     * filename property. If $param is empty and file is  
     * open, do nothing. */
    function open($new = false) {
        $this->error = '';
        $file   = $this->filename;
        $create = $this->create;
  
        /* Return true is file is open and $new is unset */
        if($this->filehandle && !$new) {
            return true;
        }
  
        /* Check that new file exitsts */
        if((!(file_exists($file) && is_readable($file))) && !$create) {
            return $this->set_error("$file: " . _("No such file or directory"));
        }
  
        /* Close old file, if any */
        if($this->filehandle) { $this->close(); }
        
        /* Open file. First try to open for reading and writing,
         * but fall back to read only. */
        umask($this->umask);
        $fh = @fopen($file, 'a+');
        if($fh) {
            $this->filehandle = &$fh;
            $this->filename   = $file;
            $this->writeable  = true;
        } else {
            $fh = @fopen($file, 'r');
            if($fh) {
                $this->filehandle = &$fh;
                $this->filename   = $file;
                $this->writeable  = false;
            } else {
                return $this->set_error("$file: " . _("Open failed"));
            }
        }
        return true;
    }

    /* Close the file and forget the filehandle */
    function close() {
        @fclose($this->filehandle);
        $this->filehandle = 0;
        $this->filename   = '';
        $this->writable   = false;
    }

    /* Lock the datafile - try 20 times in 5 seconds */
    function lock() {
        for($i = 0 ; $i < 20 ; $i++) {
            if(flock($this->filehandle, 2 + 4)) 
                return true;
            else
                usleep(250000);
        }
        return false;
    }

    /* Lock the datafile */
    function unlock() {
        return flock($this->filehandle, 3);
    }

    /* Overwrite the file with data from $rows
     * NOTE! Previous locks are broken by this function */
    function overwrite(&$rows) {
        $this->unlock();
        $newfh = @fopen($this->filename.'.tmp', 'w');

        if(!$newfh) {
            return $this->set_error($this->filename. '.tmp:' . _("Open failed"));
        }
        
        for($i = 0, $cnt=sizeof($rows) ; $i < $cnt ; $i++) {
            if(is_array($rows[$i])) {
                for($j = 0, $cnt_part=count($rows[$i]) ; $j < $cnt_part ; $j++) {
                    $rows[$i][$j] = $this->quotevalue($rows[$i][$j]);
                }
                $tmpwrite = sq_fwrite($newfh, join('|', $rows[$i]) . "\n");
                if ($tmpwrite == FALSE) {
                    return $this->set_error($this->filename . '.tmp:' . _("Write failed"));
                }
            }
        }       

        fclose($newfh);
        if (!@copy($this->filename . '.tmp' , $this->filename)) {
          return $this->set_error($this->filename . ':' . _("Unable to update"));
        }
        @unlink($this->filename . '.tmp');
	chmod($this->filename, 0600);
        $this->unlock();
        $this->open(true);
        return true;
    }
    
    /* ========================== Public ======================== */
    
    /* Search the file */
    function search($expr,$field) {
	global $fieldarrayabook;
        /* To be replaced by advanded search expression parsing */
        if(is_array($expr)) { return false; }
  
        /* Make regexp from glob'ed expression
         * May want to quote other special characters like (, ), -, [, ], etc. */
        $expr = str_replace('?', '.', $expr);
        $expr = str_replace('*', '.*', $expr);
  
        $res = array();
        if(!$this->open()) {
            return false;
        }
        @rewind($this->filehandle);
        while ($row = @fgetcsv($this->filehandle, 2048, '|')) {
	    if($expr !="" && eregi($expr,$row[$fieldarrayabook[$field]['idfile']])) {
		$arow = array();
		foreach ($fieldarrayabook as $k => $id) {
		    $arow[$k] = $row[$id['idfile']];
		}
		$arow['backend'] = $this->bnum;
		$arow['source'] = &$this->sname;
		array_push($res,$arow);
            }
        }
        return $res;
    }
    
    /* Lookup alias */
    function test_id($userdata) {
	global $fieldarrayabook;
	if (!empty($userdata['nickname'])) { $alias = strtolower($userdata['nickname']); }
	else { $alias=""; }
        $this->open();
        @rewind($this->filehandle);
	$idmax = 0;
        while ($row = @fgetcsv($this->filehandle, 2048, '|')) {
	    if ($row[$fieldarrayabook["idrub"]['idfile']] > $idmax) { $idmax=$row[$fieldarrayabook["idrub"]['idfile']]; }
            if (!empty($alias) && strtolower($row[$fieldarrayabook["nickname"]['idfile']]) == $alias && (!$userdata['idrub'] || $userdata['idrub'] != $row[$fieldarrayabook["idrub"]['idfile']])) {
                $arow = array();
                foreach ($fieldarrayabook as $k => $id) {
                    $arow[$k] = $row[$id['idfile']];
                }
                $arow['backend'] = $this->bnum;
                $arow['source'] = &$this->sname;
		return array($arow,'');
            }
        }
        return array('',$idmax);
    }

    /* Lookup alias */
    function lookup($alias) {
	global $fieldarrayabook;
	if (!$alias) { return array(''); }
        $this->open();
        @rewind($this->filehandle);
        while ($row = @fgetcsv($this->filehandle, 2048, '|')) {
            if (strtolower($row[$fieldarrayabook["nickname"]['idfile']]) == strtolower($alias)) {
                $arow = array();
                foreach ($fieldarrayabook as $k => $id) {
                    $arow[$k] = $row[$id['idfile']];
                }
                $arow['backend'] = $this->bnum;
                $arow['source'] = &$this->sname;
		return $arow;
            }
        }
        return array('');
    }

    /* Lookup alias */
    function lookup_id($alias) {
	global $fieldarrayabook;
        $this->open();
        @rewind($this->filehandle);
        while ($row = @fgetcsv($this->filehandle, 2048, '|')) {
            if (strtolower($row[$fieldarrayabook["idrub"]['idfile']]) == strtolower($alias)) {
                $arow = array();
                foreach ($fieldarrayabook as $k => $id) {
                    $arow[$k] = $row[$id['idfile']];
                }
                $arow['backend'] = $this->bnum;
                $arow['source'] = &$this->sname;
		return $arow;
            }
        }
        return array('');
    }

    /* Lookup contacts for group */
    function lookup_group_id($strings,$address = 0, $where = "ALL") {
	global $fieldarrayabook;
	$ids = explode(",",$strings);
        $this->open();
        @rewind($this->filehandle);
	$arow = array();
        while ($row = @fgetcsv($this->filehandle, 2048, '|')) {
	    foreach ($ids as $id) {
        	if ($row[$fieldarrayabook["idrub"]['idfile']] == $id) {
		    if ($address == 1) {
			array_push($arow,'"'.$row[$fieldarrayabook['Nominativo']['idfile']].'" <'.$row[$fieldarrayabook['Email']['idfile']].'>');
		    } else if ($address == 2) {
			$a = array();
			foreach ($fieldarrayabook as $k => $id) {
			    $a[$k] = $row[$id['idfile']];
			}
			$a['backend'] = $this->bnum;
			$a['source'] = &$this->sname;
			if ($a['idrub'] && ($where != "ALL" && preg_match('/^'.$where.'/i',$a['Nominativo']) || $where == "ALL")) {
			    array_push($arow,$a);
			}
		    } else {
            		$arow["1;".$id] = $row[$fieldarrayabook['Nominativo']['idfile']];
		    }
		    break;
		}
            }
        }
        return $arow;
    }

    /* List all addresses */
    function list_addr($Email=0,$initial="ALL",$nsearch="") {
	global $fieldarrayabook;
        $res = array();
        $this->open();
        @rewind($this->filehandle);
        while ($row = @fgetcsv($this->filehandle, 2048, '|')) {
	    if ($initial == "ALL" || ($initial == "number" && preg_match('/^[0-9]/',$row[$fieldarrayabook["Nominativo"]['idfile']])) || preg_match('/^'.$initial.'/i',$row[$fieldarrayabook["Nominativo"]['idfile']])) {
		if (!$nsearch || preg_match('/'.$nsearch.'/i',$row[$fieldarrayabook["Nominativo"]['idfile']])) {
		    $arow = array();
		    foreach ($fieldarrayabook as $k => $id) {
			$arow[$k] = $row[$id['idfile']];
		    }
		    $arow['backend'] = $this->bnum;
		    $arow['source'] = &$this->sname;
		    if ($arow['idrub']) {
			array_push($res,$arow);
		    }
		}
	    }
        }
        return $res;
    }

    /* Add address */
    function add($userdata) {
	global $fieldarrayabook;
        if(!$this->writeable) {
            return $this->set_error(_("Addressbook is read-only"));
        }
        /* See if user exists already */
        $return = $this->test_id($userdata);
	$ret=$return[0];
	$idmax=$return[1] + 1;    
        if(!empty($ret)) {
            return $this->set_error(sprintf(_("Nickname '%s' already exist"),
                   $ret['nickname']));
        }
	$arow = array();
	foreach ($fieldarrayabook as $k => $id) {
	    $arow[$id['idfile']] = $k;
	}
	$lastk = count($arow);
	ksort ($arow);
	reset ($arow);
	$lastk = count($arow);
	$data = "";
	foreach ($arow as $k => $id) {
	    if ($id == "idrub") { $data .= $idmax; } 
	    else { $data .= $this->quotevalue($userdata[$id]); }
	    if ($k != $lastk) { $data .= '|'; }
	}

        /* Strip linefeeds */
        $data = ereg_replace("[\r\n]", ' ', $data);
        /* Add linefeed at end */
        $data = $data . "\n";
  
        /* Reopen file, just to be sure */
        $this->open(true);
        if(!$this->writeable) {
            return $this->set_error(_("Addressbook is read-only"));
        }
  
        /* Lock the file */
        if(!$this->lock()) {
            return $this->set_error(_("Could not lock datafile"));
        }
  
        /* Write */
        $r = sq_fwrite($this->filehandle, $data);

        /* Unlock file */
        $this->unlock();

	chmod($this->filename, 0600);  

        /* Test write result and exit if OK */
        if($r == FALSE) {
	    /* Fail */
    	    $this->set_error(_("Write to addressbook failed"));
    	    return false;
	}
	return true;
    }

    /* Delete address */
    function remove($alias) {
	global $fieldarrayabook;
        if(!$this->writeable) {
            return $this->set_error(_("Addressbook is read-only"));
        }
        
        /* Lock the file to make sure we're the only process working
         * on it. */
        if(!$this->lock()) {
            return $this->set_error(_("Could not lock datafile"));
        }
  
        /* Read file into memory, ignoring nicknames to delete */
        @rewind($this->filehandle);
        $i = 0;
        $rows = array();
        while($row = @fgetcsv($this->filehandle, 2048, '|')) {
            if($row[$fieldarrayabook["idrub"]['idfile']] != $alias) {
                $rows[$i++] = $row;
            }
        }
  
        /* Write data back */
        if(!$this->overwrite($rows)) {
            $this->unlock();
            return false;
        }
  
        $this->unlock();
        return true;
    }

    /* Modify address */
    function modify($alias, $userdata) {
	global $fieldarrayabook;
        if(!$this->writeable) {
            return $this->set_error(_("Addressbook is read-only"));
        }
  
        /* See if user exists */
        $ret = $this->lookup($alias);
        if(empty($ret)) {
            return $this->set_error(sprintf(_("User '%s' does not exist"),
                $userdata['Nominativo']));
        }
	
        /* See if user exists already */
        $return = $this->test_id($userdata);
	$ret=$return[0];
	$idmax=$return[1] + 1;    
        if(!empty($ret)) {
            return $this->set_error(sprintf(_("Nickname '%s' already exist"),
                   $ret['nickname']));
        }
  
        /* Lock the file to make sure we're the only process working
         * on it. */
        if(!$this->lock()) {
            return $this->set_error(_("Could not lock datafile"));
        }
  
        /* Read file into memory, modifying the data for the 
         * user identified by $alias */
        $this->open(true);
        @rewind($this->filehandle);
        $i = 0;
        $rows = array();
        while($row = @fgetcsv($this->filehandle, 2048, '|')) {
            if(strtolower($row[$fieldarrayabook["idrub"]['idfile']]) != strtolower($alias)) {
                $rows[$i++] = $row;
            } else {
		$arow = array();
		foreach ($fieldarrayabook as $k => $id) {
		    $arow[$id['idfile']] = $userdata[$k];
		}
		$rows[$i++] = $arow;
            }
        }
  
        /* Write data back */
        if(!$this->overwrite($rows)) {
            $this->unlock();
            return false;
        }
  
        $this->unlock();
        return true;
    }
     
    /* Function for quoting values before saving */
    function quotevalue($value) {
        /* Quote the field if it contains | or ". Double quotes need to
         * be replaced with "" */
        if(ereg("[|\"]", $value)) {
            $value = '"' . str_replace('"', '""', $value) . '"';
        }
        return $value;
    }

} /* End of class abook_local_file */
?>