<?php

/********************************************************************
 *   Autori:             Marco Colombo (marco@openit.it)
 *                       Valentina Portolan (valentina@openit.it)
 *                       Giuliano Natali Diaolin (diaolin@openit.it)
 *  Copyright:           Open It S.r.l.
 *                       Viale Dante, 78
 *                       38057 Pergine Valsugana (TN) ITALY
 *                       Tel: +39 0461 534800 Fax: +39 0461 538443
 *********************************************************************/

/**
 * addressbook.php
 *
 * Copyright (c) 1999-2003 The SquirrelMail Project Team
 * Licensed under the GNU GPL. For full terms see the file COPYING.
 *
 * Functions and classes for the addressbook system.
 *
 * $Id: addressbook.php,v 1.11 2004/06/09 09:00:36 marco Exp $
 */

/*
   This is the path to the global site-wide addressbook.
   It looks and feels just like a user's .abook file
   If this is in the data directory, use "$data_dir/global.abook"
   If not, specify the path as though it was accessed from the
   src/ directory ("../global.abook" -> in main directory)

   If you don't want a global site-wide addressbook, comment these
   two lines out.  (They are disabled by default.)

   The global addressbook is unmodifiable by anyone.  You must actually
   use a shell script or whatnot to modify the contents.
  $address_book_global_filename = $data_dir."global";

*/

if (!defined('SM_PATH')) { define('SM_PATH','../'); }

include_once('global.php'); 

global $addrbook_dsn, $data_dir, $address_book_global_filename, $group_address_book_global_filename;
sqgetGlobalVar('address_book_global_filename',  $address_book_global_filename,      		SQ_SESSION); 
sqgetGlobalVar('group_address_book_global_filename',  $group_address_book_global_filename,      SQ_SESSION); 

/*
   Create and initialize an addressbook object.
   Returns the created object
*/
function addressbook_init($showerr = true, $onlylocal = false, $user = "") {
    global $data_dir, $username, $ldap_server, $address_book_global_filename, $group_address_book_global_filename;
    global $addrbook_dsn, $addrbook_table, $group_addrbook_table;
    
    if (!$user) { $user = $username; }

    /* Create a new addressbook object */
    $abook = new AddressBook;
    $gabook = new GroupAddressBook;
    // caso di lettura addressbook da database
    if (isset($addrbook_dsn) && !empty($addrbook_dsn)) {
	// tabella contatti
        if (!isset($addrbook_table) || empty($addrbook_table)) {
            $addrbook_table = 'address';
        }
	// tabella gruppi contatti
        if (!isset($group_addrbook_table) || empty($group_addrbook_table)) {
            $group_addrbook_table = 'group_address';
        }

	// carica contatti personali
        $r = $abook->add_backend('database_local', Array('dsn' => $addrbook_dsn,
                            'owner' => $user,
                            'table' => $addrbook_table));
        if (!$r && $showerr) {
            echo codeHTML(_("Error initializing addressbook personal database.")."\n");
	    echo codeHTML($abook->error);
    	    exit;
        }

	// carica contatti globali
        $r = $abook->add_backend('database_global', Array('dsn' => $addrbook_dsn,
                            'table' => $addrbook_table));
        if (!$r && $showerr) {
            echo codeHTML(_("Error initializing addressbook global database.")."\n");
            echo codeHTML($abook->error);
            exit;
        }

	// carica gruppi contatti personali
        $r = $gabook->add_backend('database_local', Array('dsn' => $addrbook_dsn,
                            'owner' => $user,
                            'table' => $group_addrbook_table));
        if (!$r && $showerr) {
            echo codeHTML(_("Error initializing group addressbook personal database.")."\n");
	    echo codeHTML($gabook->error);
    	    exit;
        }

	// carica gruppi contatti globali
        $r = $gabook->add_backend('database_global', Array('dsn' => $addrbook_dsn,
                            'table' => $group_addrbook_table));
        if (!$r && $showerr) {
            echo codeHTML(_("Error initializing group addressbook global database.")."\n");
            echo codeHTML($gabook->error);
            exit;
        }

    // caso di lettura addressbook da files
    } else {

	// carica contatti personali
        $filename = getHashedFile($user, $data_dir, "$user.abook");
        $r = $abook->add_backend('local_file', array('filename' => $filename,
                              'create'   => true));
        if(!$r && $showerr) {
            echo codeHTML(sprintf( _("Error opening file %s"), $filename ));
            exit;
        }
	// carica contatti globali
	if (!empty($address_book_global_filename)) {
    	    $r = $abook->add_backend('global_file');
    	    if (!$r && $showerr) {
        	echo codeHTML(_("Error initializing global addressbook."));
        	exit;
    	    }
	}

	// carica gruppi contatti personali
        $filename = getHashedFile($user, $data_dir, "$user.group");
        $r = $gabook->add_backend('local_file', array('filename' => $filename,
                              'create'   => true));
        if(!$r && $showerr) {
            echo codeHTML(sprintf( _("Error opening file %s"), $filename ));
            exit;
        }

	// carica gruppi contatti globali
	if (!empty($group_address_book_global_filename)) {
    	    $r = $gabook->add_backend('global_file');
    	    if (!$r && $showerr) {
        	echo codeHTML(_("Error initializing global group addressbook."));
        	exit;
    	    }
	} 
    } 

    if ($onlylocal) { return array($abook,$gabook); }
    return array($abook,$gabook);
}

/* Sort array by the key "name" */
function alistcmp($a,$b) {
    if ($a['backend'] > $b['backend']) {
        return 1;
    } else {
        if ($a['backend'] < $b['backend']) {
            return -1;
        }
    }
    return (strtolower($a['name']) > strtolower($b['name'])) ? 1 : -1;
}

/*
 *   Had to move this function outside of the Addressbook Class
 *   PHP 4.0.4 Seemed to be having problems with inline functions.
 */    
function addressbook_cmp($a,$b) {

    if($a['backend'] > $b['backend']) {
        return 1;
    } else if($a['backend'] < $b['backend']) {
        return -1;
    }
    
    return (strtolower($a['name']) > strtolower($b['name'])) ? 1 : -1;

}

/*
 * This is the main address book class that connect all the
 * backends and provide services to the functions above.
 *
 */

class AddressBook {

    var $backends    = array();
    var $numbackends = 0;
    var $error       = '';
    var $localbackend = 0;
    var $localbackendname = '';
    
      // Constructor function.
    function AddressBook() {
        $localbackendname = _("Personal address book");
    }

    /*
     * Return an array of backends of a given type,
     * or all backends if no type is specified.
     */
    function get_backend_list($type = '') {
        $ret = array();
        for ($i = 1 ; $i <= $this->numbackends ; $i++) {
            if (empty($type) || $type == $this->backends[$i]->btype) {
                $ret[] = &$this->backends[$i];
            }
        }
        return $ret;
    }


    /*
       ========================== Public ========================

        Add a new backend. $backend is the name of a backend
        (without the abook_ prefix), and $param is an optional
        mixed variable that is passed to the backend constructor.
        See each of the backend classes for valid parameters.
     */
    function add_backend($backend, $param = '') {
        $backend_name = 'abook_' . $backend;
        eval('$newback = new ' . $backend_name . '($param);');
        if(!empty($newback->error)) {
            $this->error = $newback->error;
            return false;
        }

        $this->numbackends++;

        $newback->bnum = $this->numbackends;
        $this->backends[$this->numbackends] = $newback;
        
        /* Store ID of first local backend added */
        if ($this->localbackend == 0 && $newback->btype == 'local') {
            $this->localbackend = $this->numbackends;
            $this->localbackendname = $newback->sname;
        }

        return $this->numbackends;
    }


    /*
     * This function takes a $row array as returned by the addressbook 
     * search and returns an e-mail address with the full name or
     * nickname optionally prepended.
     */

    function full_address($row) {
        global $addrsrch_fullname, $data_dir, $username;

        $prefix = getPref($data_dir, $username, 'addrsrch_fullname');
        if (($prefix != "" || (isset($addrsrch_fullname) && $prefix == $addrsrch_fullname)) && $prefix != 'noprefix') {
            $name = ($prefix == 'nickname' ? $row['nickname'] : $row['Nominativo']);
            return $name . ' <' . trim($row['Email']) . '>';
        } else {
            return trim($row['Email']);
        }
    }

    /*
        Return a list of addresses matching expression in
        all backends of a given type.
    */
    function search($expression, $field, $bnum = -1) {
        $ret = array();
        $this->error = '';

        /* Search all backends */
        if ($bnum == -1) {
            $sel = $this->get_backend_list('');
            $failed = 0;
            for ($i = 0 ; $i < sizeof($sel) ; $i++) {
                $backend = &$sel[$i];
                $backend->error = '';
                $res = $backend->search($expression,$field);
                if (is_array($res)) {
                    $ret = array_merge($ret, $res);
                } else {
                    $this->error .= "<br>\n" . $backend->error;
                    $failed++;
                }
            }

            /* Only fail if all backends failed */
            if( $failed >= sizeof( $sel ) ) {
                $ret = FALSE;
            }

        }  else {

            /* Search only one backend */

            $ret = $this->backends[$bnum]->search($expression,$field);
            if (!is_array($ret)) {
                $this->error .= "<br>\n" . $this->backends[$bnum]->error;
                $ret = FALSE;
            }
        }

        return( $ret );
    }


    /* Return a sorted search */
    function s_search($expression, $field, $bnum = -1) {
    
        $ret = $this->search($expression, $field, $bnum);
        if ( is_array( $ret ) ) {
            usort($ret, 'addressbook_cmp');
        }    
        return $ret;
    }


    /*
     *  Lookup an address by alias. Only possible in
     *  local backends.
     */
    function lookup($alias, $bnum = -1) {
    
        $ret = array();
	// specifico il backend dove cercare
        if ($bnum > -1) {
            $res = $this->backends[$bnum]->lookup($alias);
            if (is_array($res)) {
               return $res;
            } else {
               $this->error = $backend->error;
               return false;
            }
        }
	// cerco su tutti i backends    
        $sel = $this->get_backend_list('');
        for ($i = 0 ; $i < sizeof($sel) ; $i++) {
            $backend = &$sel[$i];
            $backend->error = '';
            $res = $backend->lookup($alias);
            if (is_array($res)) {
               if(!empty($res))
              return $res;
            } else {
               $this->error = $backend->error;
               return false;
            }
        }
        
        return $ret;
    }

    function lookup_id($alias, $bnum = -1) {
    
        $ret = array();
	// specifico il backend dove cercare
        if ($bnum > -1) {
            $res = $this->backends[$bnum]->lookup_id($alias);
            if (is_array($res)) {
               return $res;
            } else {
               $this->error = $backend->error;
               return false;
            }
        }
	// cerco su tutti i backends
        $sel = $this->get_backend_list('');
        for ($i = 0 ; $i < sizeof($sel) ; $i++) {
            $backend = &$sel[$i];
            $backend->error = '';
            $res = $backend->lookup_id($alias);
            if (is_array($res)) {
               if(!empty($res))
              return $res;
            } else {
               $this->error = $backend->error;
               return false;
            }
        }
        
        return $ret;
    }

    function lookup_group_id($string, $bnum = -1, $address = 0, $where = "ALL") {
        $ret = array();
	// specifico il backend dove cercare
        if ($bnum > -1) {
            $res = $this->backends[$bnum]->lookup_group_id($string, $address, $where);
            if (is_array($res)) {
               return $res;
            } else {
               $this->error = $backend->error;
               return false;
            }
        }
    }

    /* Return all addresses */
    function list_addr($bnum = -1,$Email= 0,$initial = "ALL",$nsearch="") {
        $ret = array();
        // cerco su tutti i backends
        if ($bnum == -1) {
            $sel = $this->get_backend_list('');
	// specifico il backend dove cercare
        } else {
            $sel = array(0 => &$this->backends[$bnum]);
        }
        for ($i = 0 ; $i < sizeof($sel) ; $i++) {
            $backend = &$sel[$i];
            $backend->error = '';
    	    $res = $backend->list_addr($Email,$initial,$nsearch);
            if (is_array($res)) {
               $ret = array_merge($ret, $res);
            } else {
               $this->error = $backend->error;
               return false;
            }
        }
        
        return $ret;
    }

    /*
     * Create a new address from $userdata, in backend $bnum.
     * Return the backend number that the/ address was added
     * to, or false if it failed.
     */
    function add($userdata, $bnum) {
        /* Validate data */
        if (!is_array($userdata)) {
            $this->error = _("Invalid input data");
            return false;
        }
        if (empty($userdata['Nominativo'])) {
            $this->error = _("Name is missing");
            return false;
        }
        if (empty($userdata['Email'])) {
            $this->error = _("E-mail address is missing");
            return false;
        }
        if (!empty($userdata['nickname'])) {
    	    if (eregi('[ \\:\\|\\#\\"\\!]', $userdata['nickname'])) {
        	$this->error = _("Nickname contains illegal characters");
        	return false;
    	    }
        }

        /* Check that specified backend accept new entries */
        if (!$this->backends[$bnum]->writeable) {
            $this->error = _("Addressbook is read-only");
            return false;
        }

	if ($bnum == 2) {
	    $userdata['nickname']="";
	} else {
    	    /* Test id su privato */
    	    $res = $this->backends[$bnum]->test_id($userdata);
    	    if (!empty($res["idrub"])) {
		$this->error = sprintf(_("Nickname '%s' already exist"),$userdata['nickname']);
        	return false;
    	    }
	}    

        /* Add address to backend */
        $res = $this->backends[$bnum]->add($userdata);
        if ($res) {
            return $bnum;
        } else {
            $this->error = $this->backends[$bnum]->error;
            return false;
        }
    } /* end of add() */


    /*
     * Remove the user identified by $alias from backend $bnum
     * If $alias is an array, all users in the array are removed.
     */
    function remove($alias, $bnum) {
    
        /* Check input */
        if (empty($alias)) { return true; }
        
        /* Check that specified backend is writable */
        if (!$this->backends[$bnum]->writeable) {
            $this->error = _("Addressbook is read-only");
            return false;
        }
        
        /* Remove user from backend */
        $res = $this->backends[$bnum]->remove($alias);
        if ($res) {
            return $bnum;
        } else {
            $this->error = $this->backends[$bnum]->error;
            return false;
        }
        
        return FALSE;  /* Not reached */
    } /* end of remove() */


    /*
     * Remove the user identified by $alias from backend $bnum
     * If $alias is an array, all users in the array are removed.
     */
    function modify($alias, $userdata, $bnum) {
    
        /* Check input */
        if (empty($alias)) { return true; }
        
        /* Validate data */
        if(!is_array($userdata)) {
            $this->error = _("Invalid input data");
            return false;
        }
        if (empty($userdata['Nominativo'])) {
            $this->error = _("Name is missing");
            return false;
        }
        if (empty($userdata['Email'])) {
            $this->error = _("E-mail address is missing");
            return false;
        }
        
        if (!empty($userdata['nickname'])) {
    	    if (eregi('[\\: \\|\\#"\\!]', $userdata['nickname'])) {
        	$this->error = "*".$userdata['nickname']."*"._("Nickname contains illegal characters");
        	return false;
	    }
        }
        
        /* Check that specified backend is writable */
        if (!$this->backends[$bnum]->writeable) {
            $this->error = _("Addressbook is read-only");;
            return false;
        }
        if ($bnum == 2) {
	    $userdata['nickname']="";
	} else {	
    	    /* Test id su privato */
    	    $res = $this->backends[1]->test_id($userdata,$alias);
    	    if (!empty($res["idrub"])) {
		$this->error = sprintf(_("Nickname '%s' already exist"),$userdata['nickname']);   
        	return false;
    	    }
	}

        /* Modify user in backend */
        $res = $this->backends[$bnum]->modify($alias, $userdata);
        if ($res) {
            return $bnum;
        } else {
            $this->error = $this->backends[$bnum]->error;
            return false;
        }
        
        return FALSE;  /* Not reached */
    } /* end of modify() */
    
    
} /* End of class Addressbook */

/*
 * This is the main address book class that connect all the
 * backends and provide services to the functions above.
 *
 */

class GroupAddressBook {

    var $backends    = array();
    var $numbackends = 0;
    var $error       = '';
    var $localbackend = 0;
    var $localbackendname = '';
    
      // Constructor function.
    function GroupAddressBook() {
        $localbackendname = _("Personal group address book");
    }

    /*
     * Return an array of backends of a given type,
     * or all backends if no type is specified.
     */
    function get_backend_list($type = '') {
        $ret = array();
        for ($i = 1 ; $i <= $this->numbackends ; $i++) {
            if (empty($type) || $type == $this->backends[$i]->btype) {
                $ret[] = &$this->backends[$i];
            }
        }
        return $ret;
    }


    /*
       ========================== Public ========================

        Add a new backend. $backend is the name of a backend
        (without the abook_ prefix), and $param is an optional
        mixed variable that is passed to the backend constructor.
        See each of the backend classes for valid parameters.
     */
    function add_backend($backend, $param = '') {
        $backend_name = 'group_abook_' . $backend;
        eval('$newback = new ' . $backend_name . '($param);');
        if(!empty($newback->error)) {
            $this->error = $newback->error;
            return false;
        }

        $this->numbackends++;

        $newback->bnum = $this->numbackends;
        $this->backends[$this->numbackends] = $newback;
        
        /* Store ID of first local backend added */
        if ($this->localbackend == 0 && $newback->btype == 'local') {
            $this->localbackend = $this->numbackends;
            $this->localbackendname = $newback->sname;
        }

        return $this->numbackends;
    }

    /*
        Return a list of addresses matching expression in
        all backends of a given type.
    */
    function search($expression, $field, $bnum = -1) {
        $ret = array();
        $this->error = '';

        /* Search all backends */
        if ($bnum == -1) {
	    // cerco su tutti i backends
            $sel = $this->get_backend_list('');
            $failed = 0;
            for ($i = 0 ; $i < sizeof($sel) ; $i++) {
                $backend = &$sel[$i];
                $backend->error = '';
                $res = $backend->search($expression,$field);
                if (is_array($res)) {
                    $ret = array_merge($ret, $res);
                } else {
                    $this->error .= "<br>\n" . $backend->error;
                    $failed++;
                }
            }

            /* Only fail if all backends failed */
            if( $failed >= sizeof( $sel ) ) {
                $ret = FALSE;
            }

        }  else {

	    // specifico il backend dove cercare
            $ret = $this->backends[$bnum]->search($expression,$field);
            if (!is_array($ret)) {
                $this->error .= "<br>\n" . $this->backends[$bnum]->error;
                $ret = FALSE;
            }
        }

        return( $ret );
    }


    /* Return a sorted search */
    function s_search($expression, $field, $bnum = -1) {
    
        $ret = $this->search($expression, $field, $bnum);
        if ( is_array( $ret ) ) {
            usort($ret, 'addressbook_cmp');
        }    
        return $ret;
    }


    /*
     *  Lookup an address by alias. Only possible in
     *  local backends.
     */
    function lookup($alias, $bnum = -1) {
    
        $ret = array();
	// specifico il backend dove cercare
        if ($bnum > -1) {
            $res = $this->backends[$bnum]->lookup($alias);
            if (is_array($res)) {
               return $res;
            } else {
               $this->error = $backend->error;
               return false;
            }
        }
	// cerco su tutti i backends
        $sel = $this->get_backend_list('');
        for ($i = 0 ; $i < sizeof($sel) ; $i++) {
            $backend = &$sel[$i];
            $backend->error = '';
            $res = $backend->lookup($alias);
            if (is_array($res)) {
               if(!empty($res))
              return $res;
            } else {
               $this->error = $backend->error;
               return false;
            }
        }
        
        return $ret;
    }

    function lookup_id($alias, $bnum = -1) {
    
        $ret = array();
	// specifico il backend dove cercare
        if ($bnum > -1) {
            $res = $this->backends[$bnum]->lookup_id($alias);
            if (is_array($res)) {
               return $res;
            } else {
               $this->error = $backend->error;
               return false;
            }
        }
	// cerco su tutti i backends
        $sel = $this->get_backend_list('');
        for ($i = 0 ; $i < sizeof($sel) ; $i++) {
            $backend = &$sel[$i];
            $backend->error = '';
            $res = $backend->lookup_id($alias);
            if (is_array($res)) {
               if(!empty($res))
              return $res;
            } else {
               $this->error = $backend->error;
               return false;
            }
        }
        
        return $ret;
    }

    /* Return all addresses */
    function list_addr($bnum = -1) {
        $ret = array();
	// cerco su tutti i backends
        if ($bnum == -1) {
            $sel = $this->get_backend_list();
        // specifico il backend dove cercare
        } else {
            $sel = array(0 => &$this->backends[$bnum]);
        }
        for ($i = 0 ; $i < sizeof($sel) ; $i++) {
            $backend = &$sel[$i];
            $backend->error = '';
    	    $res = $backend->list_addr();
            if (is_array($res)) {
               $ret = array_merge($ret, $res);
            } else {
               $this->error = $backend->error;
               return false;
            }
        }
        return $ret;
    }

    /*
     * Create a new address from $userdata, in backend $bnum.
     * Return the backend number that the/ address was added
     * to, or false if it failed.
     */
    function add($userdata, $bnum) {
        /* Validate data */
        if (!is_array($userdata)) {
            $this->error = _("Invalid input data");
            return false;
        }
        if (empty($userdata['group_name'])) {
            $this->error = _("Group Name is missing");
            return false;
        }

        /* Check that specified backend accept new entries */
        if (!$this->backends[$bnum]->writeable) {
            $this->error = _("Group Addressbook is read-only");
            return false;
        }

	if ($bnum == 2) {
    	    /* Test id su pubblico */
    	    $res = $this->backends[$bnum]->test_id($userdata);
    	    if (!empty($res["group_id"])) {
		$this->error = sprintf(_("Group Name '%s' already exist"),$userdata['group_name']);
        	return false;
    	    }
	} else {
    	    /* Test id su privato */
    	    $res = $this->backends[$bnum]->test_id($userdata);
    	    if (!empty($res["group_id"])) {
		$this->error = sprintf(_("Group Name '%s' already exist"),$userdata['group_name']);
        	return false;
    	    }
	}

        /* Add address to backend */
        $res = $this->backends[$bnum]->add($userdata);
        if ($res) {
            return $bnum;
        } else {
            $this->error = $this->backends[$bnum]->error;
            return false;
        }
    } /* end of add() */


    /*
     * Remove the user identified by $alias from backend $bnum
     * If $alias is an array, all users in the array are removed.
     */
    function remove($alias, $bnum) {
    
        /* Check input */
        if (empty($alias)) { return true; }
        
        /* Check that specified backend is writable */
        if (!$this->backends[$bnum]->writeable) {
            $this->error = _("Addressbook is read-only");
            return false;
        }
        
        /* Remove user from backend */
        $res = $this->backends[$bnum]->remove($alias);
        if ($res) {
            return $bnum;
        } else {
            $this->error = $this->backends[$bnum]->error;
            return false;
        }
        
        return FALSE;  /* Not reached */
    } /* end of remove() */

    /*
     * Remove the user identified by $alias from backend $bnum
     * If $alias is an array, all users in the array are removed.
     */
    function remove_contacts($alias,$bnum=1,$public=0) {
        /* Check input */
        if (empty($alias)) { return true; }
    	if (!$this->backends[$bnum]->writeable) {
    	    $this->backends[$bnum]->error = _("Addressbook is read-only");
    	    return false;
    	}
	if ($public) {
    	    $res = $this->backends[$bnum]->remove_contacts($alias,1);
	} else {
    	    $res = $this->backends[$bnum]->remove_contacts($alias);
	}
    	if (!$res) {
    	    $this->error = $this->backends[$bnum]->error;
	    return $this->error;
	}
        return FALSE;
    }


    /*
     * Remove the user identified by $alias from backend $bnum
     * If $alias is an array, all users in the array are removed.
     */
    function modify($alias, $userdata, $bnum) {
    
        /* Check input */
        if (empty($alias)) { return true; }
        
        /* Validate data */
        if(!is_array($userdata)) {
            $this->error = _("Invalid input data");
            return false;
        }
        if (empty($userdata['group_name'])) {
            $this->error = _("Group Name is missing");
            return false;
        }
        
        /* Check that specified backend is writable */
        if (!$this->backends[$bnum]->writeable) {
            $this->error = _("Group Addressbook is read-only");;
            return false;
        }
        if ($bnum == 2) {
    	    /* Test id su pubblico */
    	    $res = $this->backends[$bnum]->test_id($userdata,$alias);
    	    if (!empty($res["group_id"])) {
		$this->error = sprintf(_("Group Name '%s' already exist"),$userdata['group_name']);   
        	return false;
    	    }
	} else {	
    	    /* Test id su privato */
    	    $res = $this->backends[$bnum]->test_id($userdata,$alias);
    	    if (!empty($res["group_id"])) {
		$this->error = sprintf(_("Group Name '%s' already exist"),$userdata['group_name']);   
        	return false;
    	    }
	}

        /* Modify user in backend */
        $res = $this->backends[$bnum]->modify($alias, $userdata);
        if ($res) {
            return $bnum;
        } else {
            $this->error = $this->backends[$bnum]->error;
            return false;
        }
        
        return FALSE;  /* Not reached */
    } /* end of modify() */
    
    
} /* End of class Addressbook */


/*
 * Generic backend that all other backends extend
 */
class addressbook_backend {

    /* Variables that all backends must provide. */
    var $btype      = 'dummy';
    var $bname      = 'dummy';
    var $sname      = 'Dummy backend';
    
    /*
     * Variables common for all backends, but that
     * should not be changed by the backends.
     */
    var $bnum       = -1;
    var $error      = '';
    var $writeable  = false;
    
    function set_error($string) {
        $this->error = '[' . $this->sname . '] ' . $string;
        return false;
    }
    
    
    /* ========================== Public ======================== */
    
    function search($expression) {
        $this->set_error('search not implemented');
        return false;
    }
    
    function lookup($alias) {
        $this->set_error('lookup not implemented');
        return false;
    }
    
    function list_addr() {
        $this->set_error('list_addr not implemented');
        return false;
    }
    
    function add($userdata) {
        $this->set_error('add not implemented');
        return false;
    }
    
    function remove($alias) {
        $this->set_error('delete not implemented');
        return false;
    }
    
    function modify($alias, $newuserdata) {
        $this->set_error('modify not implemented');
        return false;
    }

}

/*
  PHP 5 requires that the class be made first, which seems rather
  logical, and should have been the way it was generated the first time.
*/

include_once('abook_local_file.php'); 
include_once('group_abook_local_file.php'); 

/* Use this if you wanna have a global address book */
if (isset($address_book_global_filename)) {
    include_once('abook_global_file.php'); 
    include_once('group_abook_global_file.php'); 
}

/* Only load database backend if database is configured */
if(isset($addrbook_dsn) && !empty($addrbook_dsn)) {
    include_once('abook_database_global.php'); 
    include_once('abook_database_local.php'); 
    include_once('group_abook_database_global.php'); 
    include_once('group_abook_database_local.php'); 
}

?>