<?php

/********************************************************************
 *   Autori:             Marco Colombo (marco@openit.it)
 *                       Valentina Portolan (valentina@openit.it)
 *                       Giuliano Natali Diaolin (diaolin@openit.it)
 *  Copyright:           Open It S.r.l.
 *                       Viale Dante, 78
 *                       38057 Pergine Valsugana (TN) ITALY
 *                       Tel: +39 0461 534800 Fax: +39 0461 538443
 *********************************************************************/

/**
 * options.php
 *
 * Copyright (c) 1999-2004 The SquirrelMail Project Team
 * Licensed under the GNU GPL. For full terms see the file COPYING.
 *
 * Functions needed to display the options pages.
 *
 * $Id: options.php,v 1.34.2.5 2004/03/24 16:10:59 tokul Exp $
 */

if (!defined('SM_PATH')) { define('SM_PATH','../'); }

require_once(SM_PATH . 'functions/forms.php');
require_once(SM_PATH . 'functions/global.php');

/**********************************************/
/* Define constants used in the options code. */
/**********************************************/

/* Define constants for the various option types. */
define('SMOPT_TYPE_STRING', 0);
define('SMOPT_TYPE_STRLIST', 1);
define('SMOPT_TYPE_TEXTAREA', 2);
define('SMOPT_TYPE_INTEGER', 3);
define('SMOPT_TYPE_FLOAT', 4);
define('SMOPT_TYPE_BOOLEAN', 5);
define('SMOPT_TYPE_HIDDEN', 6);
define('SMOPT_TYPE_COMMENT', 7);
define('SMOPT_TYPE_FLDRLIST', 8);

/* Define constants for the options refresh levels. */
define('SMOPT_REFRESH_NONE', 0);
define('SMOPT_REFRESH_FOLDERLIST', 1);
define('SMOPT_REFRESH_ALL', 2);

/* Define constants for the options size. */
define('SMOPT_SIZE_TINY', 0);
define('SMOPT_SIZE_SMALL', 1);
define('SMOPT_SIZE_MEDIUM', 2);
define('SMOPT_SIZE_LARGE', 3);
define('SMOPT_SIZE_HUGE', 4);
define('SMOPT_SIZE_NORMAL', 5);

define('SMOPT_SAVE_DEFAULT', 'save_option');
define('SMOPT_SAVE_NOOP', 'save_option_noop');

/**
 * SquirrelOption: An option for Squirrelmail.
 *
 * This class is a work in progress. When complete, it will handle
 * presentation and saving of Squirrelmail user options in a simple,
 * streamline manner. Stay tuned for more stuff.
 *
 * Also, I'd like to ask that people leave this alone (mostly :) until
 * I get it a little further along. That should only be a day or two or
 * three. I will remove this message when it is ready for primetime usage.
 */
class SquirrelOption {
    /* The basic stuff. */
    var $name;
    var $caption;
    var $type;
    var $refresh_level;
    var $size;
    var $comment;
    var $script;
    var $post_script;

    /* The name of the Save Function for this option. */
    var $save_function;

    /* The various 'values' for this options. */
    var $value;
    var $new_value;
    var $possible_values;

    function SquirrelOption
    ($name, $caption, $type, $refresh_level, $initial_value = '', $possible_values = '') {
        /* Set the basic stuff. */
        $this->name = $name;
        $this->caption = $caption;
        $this->type = $type;
        $this->refresh_level = $refresh_level;
        $this->possible_values = $possible_values;
        $this->size = SMOPT_SIZE_MEDIUM;
        $this->comment = '';
        $this->script = '';
        $this->post_script = '';

        /* Check for a current value. */
        if (!empty($initial_value)) {
            $this->value = $initial_value;
        } else if (isset($GLOBALS[$name])) {
            $this->value = $GLOBALS[$name];
        } else {
            $this->value = '';
        }

        /* Check for a new value. */
	if ( !sqgetGlobalVar("new_$name", $this->new_value, SQ_POST ) ) {
            $this->new_value = '';
        }

        /* Set the default save function. */
        if (($type != SMOPT_TYPE_HIDDEN) && ($type != SMOPT_TYPE_COMMENT)) {
            $this->save_function = SMOPT_SAVE_DEFAULT;
        } else {
            $this->save_function = SMOPT_SAVE_NOOP;
        }
    }

    /* Set the value for this option. */
    function setValue($value) {
        $this->value = $value;
    }

    /* Set the new value for this option. */
    function setNewValue($new_value) {
        $this->new_value = $new_value;
    }

    /* Set the size for this option. */
    function setSize($size) {
        $this->size = $size;
    }

    /* Set the comment for this option. */
    function setComment($comment) {
        $this->comment = $comment;
    }

    /* Set the script for this option. */
    function setScript($script) {
        $this->script = $script;
    }

    /* Set the "post script" for this option. */
    function setPostScript($post_script) {
        $this->post_script = $post_script;
    }

    /* Set the save function for this option. */
    function setSaveFunction($save_function) {
        $this->save_function = $save_function;
    }

    function createHTMLWidget() {
        global $javascript_on;

        /* Get the widget for this option type. */
        switch ($this->type) {
            case SMOPT_TYPE_STRING:
                $result = $this->createWidget_String();
                break;
            case SMOPT_TYPE_STRLIST:
                $result = $this->createWidget_StrList();
                break;
            case SMOPT_TYPE_TEXTAREA:
                $result = $this->createWidget_TextArea();
                break;
            case SMOPT_TYPE_INTEGER:
                $result = $this->createWidget_Integer();
                break;
            case SMOPT_TYPE_FLOAT:
                $result = $this->createWidget_Float();
                break;
            case SMOPT_TYPE_BOOLEAN:
                $result = $this->createWidget_Boolean();
                break;
            case SMOPT_TYPE_HIDDEN:
                $result = $this->createWidget_Hidden();
                break;
            case SMOPT_TYPE_COMMENT:
                $result = $this->createWidget_Comment();
                break;
            case SMOPT_TYPE_FLDRLIST:
                $result = $this->createWidget_FolderList();
                break;
            default:
               $result = '<font color="' . $color[2] . '">'
                       . codeHTML(sprintf(_("Option Type '%s' Not Found"), $this->type))
                       . '</font>';
        }

        /* Add the "post script" for this option. */
        $result .= $this->post_script;
        
        /* Now, return the created widget. */
        return ($result);
    }

    function createWidget_String() {
        switch ($this->size) {
            case SMOPT_SIZE_TINY:
                $width = 5;
                break;
            case SMOPT_SIZE_SMALL:
                $width = 12;
                break;
            case SMOPT_SIZE_LARGE:
                $width = 38;
                break;
            case SMOPT_SIZE_HUGE:
                $width = 50;
                break;
            case SMOPT_SIZE_NORMAL:
            default:
                $width = 25;
        }

	$result = addInput('new_'.$this->name, $this->value, $width, '', $this->script);
        return ($result);
    }

    function createWidget_StrList() {
	$options = array();
        /* Add each possible value to the select list. */
        foreach ($this->possible_values as $real_value => $disp_value) {
	    $options[$real_value] = $disp_value;
        }
	return addSelect("new_$this->name", $options , $this->value, TRUE, $this->script);
    }

    function createWidget_FolderList() {
        $selected = array(strtolower($this->value));

	$options = array();

        /* Begin the select tag. */

        /* Add each possible value to the select list. */
        foreach ($this->possible_values as $real_value => $disp_value) {
            if ( is_array($disp_value) ) { 
                /* For folder list, we passed in the array of boxes.. */
                $new_option = sqimap_mailbox_option_list(0, $selected, 0, $disp_value);
	        foreach ($new_option as $k => $v) {
		    $options[$k] = $v;
		}	
            } else {
		$options[$real_value] = $disp_value;
            }
        }        
	return addSelect("new_$this->name", $options , $this->value, TRUE, $this->script);
    }


    function createWidget_TextArea() {
        switch ($this->size) {
            case SMOPT_SIZE_TINY:  $rows = 3; $cols =  10; break;
            case SMOPT_SIZE_SMALL: $rows = 4; $cols =  30; break;
            case SMOPT_SIZE_LARGE: $rows = 10; $cols =  60; break;
            case SMOPT_SIZE_HUGE:  $rows = 20; $cols =  80; break;
            case SMOPT_SIZE_NORMAL:
            default: $rows = 5; $cols =  50;
        }
	$result = addTextArea('new_'.$this->name, $this->value, $cols, $rows, $this->script);
        return ($result);
    }

    function createWidget_Integer() {

        global $javascript_on;
 
        // add onChange javascript handler to a regular string widget
        // which will strip out all non-numeric chars
        if ($javascript_on)
           return preg_replace('/>/', ' onChange="origVal=this.value; newVal=\'\'; '
                    . 'for (i=0;i<origVal.length;i++) { if (origVal.charAt(i)>=\'0\' '
                    . '&& origVal.charAt(i)<=\'9\') newVal += origVal.charAt(i); } '
                    . 'this.value=newVal;">', $this->createWidget_String());
        else
           return $this->createWidget_String();
    }

    function createWidget_Float() {
        
        global $javascript_on;
 
        // add onChange javascript handler to a regular string widget
        // which will strip out all non-numeric (period also OK) chars 
        if ($javascript_on)
           return preg_replace('/>/', ' onChange="origVal=this.value; newVal=\'\'; '
                    . 'for (i=0;i<origVal.length;i++) { if ((origVal.charAt(i)>=\'0\' '
                    . '&& origVal.charAt(i)<=\'9\') || origVal.charAt(i)==\'.\') '
                    . 'newVal += origVal.charAt(i); } this.value=newVal;">'
                , $this->createWidget_String());
        else
           return $this->createWidget_String();
    }

    function createWidget_Boolean() {
        /* Do the whole current value thing. */
        if ($this->value != SMPREF_NO) {
            $yes_chk = TRUE;
            $no_chk = FALSE;
        } else {
            $yes_chk = FALSE;
            $no_chk = TRUE;
        }

        /* Build the yes choice. */
	$yes_option = addRadioBox('new_'.$this->name, $yes_chk, SMPREF_YES, $this->script).'&nbsp;'._("Yes");
        /* Build the no choice. */
	$no_option = addRadioBox('new_'.$this->name, $no_chk, SMPREF_NO, $this->script).'&nbsp;'._("No");

        /* Build and return the combined "boolean widget". */
        $result = "$yes_option&nbsp;&nbsp;&nbsp;&nbsp;$no_option";
        return ($result);
    }

    function createWidget_Hidden() {
	$result .= addHidden('new_'.$this->name, $this->value);
        return ($result);
    }

    function createWidget_Comment() {
        $result = $this->comment;
        return ($result);
    }

    function save() {
        $function = $this->save_function;
        $function($this);
    }

    function changed() {
        return ($this->value != $this->new_value);
    }
}

function save_option($option) {
    if ( !sqgetGlobalVar('username', $username, SQ_SESSION ) ) {
        /* Can't save the pref if we don't have the username */
        return;
    }
    global $data_dir;
    setPref($data_dir, $username, $option->name, $option->new_value);
}

function save_option_noop($option) {
    /* Do nothing here... */
}

function create_optpage_element($optpage) {
    return create_hidden_element('optpage', $optpage);
}

function create_optmode_element($optmode) {
    return create_hidden_element('optmode', $optmode);
}

function create_hidden_element($name, $value) {
    $result .= addHidden($name, $value);
    return ($result);
}

function create_option_groups($optgrps, $optvals) {
    /* Build a simple array with which to start. */
    $result = array();

    /* Create option group for each option group name. */
    foreach ($optgrps as $grpkey => $grpname) {
        $result[$grpkey] = array();
        $result[$grpkey]['name'] = $grpname;
        $result[$grpkey]['options'] = array();
    }

     /* Create a new SquirrelOption for each set of option values. */
    foreach ($optvals as $grpkey => $grpopts) {
        foreach ($grpopts as $optset) {
            if (isset($optset['posvals'])) {
                /* Create a new option with all values given. */
                $next_option = new SquirrelOption(
                    $optset['name'],
                    $optset['caption'],
                    $optset['type'],
		    (isset($optset['refresh']) ? $optset['refresh'] : SMOPT_REFRESH_NONE),
                    (isset($optset['initial_value']) ? $optset['initial_value'] : ''),
                    $optset['posvals']
                );
            } else {
                /* Create a new option with all but possible values given. */
                $next_option = new SquirrelOption(
                    $optset['name'],
                    $optset['caption'],
                    $optset['type'],
    	    	    (isset($optset['refresh']) ? $optset['refresh'] : SMOPT_REFRESH_NONE),
                    (isset($optset['initial_value']) ? $optset['initial_value'] : '')
                );
            }

            /* If provided, set the size for this option. */
            if (isset($optset['size'])) {
                $next_option->setSize($optset['size']);
            }

            /* If provided, set the comment for this option. */
            if (isset($optset['comment'])) {
                $next_option->setComment($optset['comment']);
            }

            /* If provided, set the save function for this option. */
            if (isset($optset['save'])) {
                $next_option->setSaveFunction($optset['save']);
            }

            /* If provided, set the script for this option. */
            if (isset($optset['script'])) {
                $next_option->setScript($optset['script']);
            }

            /* If provided, set the "post script" for this option. */
            if (isset($optset['post_script'])) {
                $next_option->setPostScript($optset['post_script']);
            }

            /* Add this option to the option array. */
            $result[$grpkey]['options'][] = $next_option;
        }
    }

    /* Return our resulting array. */
    return ($result);
}

function print_option_groups($option_groups) {
    /* Print each option group. */
    foreach ($option_groups as $next_optgrp) {
        /* If it is not blank, print the name for this option group. */
        if ($next_optgrp['name'] != '') {
            echo html_tag( 'tr', "\n".
                        html_tag( 'td',
                            '<b>' . codeHTML($next_optgrp['name']) . '</b>' ,
                        'center' ,'', 'valign="middle" colspan="2" nowrap' )
                    ) ."\n";
        }

        /* Print each option in this option group. */
        foreach ($next_optgrp['options'] as $option) {
            if ($option->type != SMOPT_TYPE_HIDDEN) {
                echo html_tag( 'tr', "\n".
                           html_tag( 'td', codeHTML($option->caption) . ':', 'right' ,'', 'valign="middle"' ) .
                           html_tag( 'td', $option->createHTMLWidget(), 'left' )
                       ) ."\n";
            } else {
                echo $option->createHTMLWidget();
            }
        }

        /* Print an empty row after this option group. */
        echo html_tag( 'tr',
                   html_tag( 'td', '&nbsp;', 'left', '', 'colspan="2"' )
                ) . "\n";
    }
}

//function OptionSubmit( $name ) {
//	echo '<tr><td colspan="2" align="right">';
//	echo addSubmit(_("Submit"), $name).'&nbsp;&nbsp;&nbsp;&nbsp;';
//	echo '</td></td>'."\n";
//}

function ShowOptionBar() {
    global $use_pop3, $base_uri, $qmlogin_cgi_url, $optpage_blocks;
    /******************************************/
    /* Build our array of Option Page Blocks. */
    /******************************************/
    $optpage_blocks = array();

    /* Build a section for Personal Options. */
    $optpage_blocks[] = array(
        'name' => _("Personal"),
	'div'  => "div1",	
        'url'  => $base_uri.'src/options.php?optpage=' . SMOPT_PAGE_PERSONAL,
        'desc' => _("This contains personal information about yourself such as your name, your email address, etc."),
        'js'   => false
    );

    do_hook('optpage_register_block');

    /* Build a section for Display Options. */
    $optpage_blocks[] = array(
        'name' => _("General"),
	'div'  => "div3",	
        'url'  => $base_uri.'src/options.php?optpage=' . SMOPT_PAGE_DISPLAY,
        'desc' => _("You can change the way that Overlook looks and displays information to you, such as the colors, the language, and other settings."),
        'js'   => false
    );

    if ($qmlogin_cgi_url) {
	/* Build a section for Display Options. */
	$optpage_blocks[] = array(
    	    'name' => _("MailAdmin"),
	    'div'  => "div4",	
    	    'url'  => $base_uri.'plugins/qmailadmin_login/options_qmailadmin_login.php',
    	    'desc' => _("Change your password, forward and vacation settings."),
    	    'js'   => false
	);
    }

    /* Build a section for Message Highlighting Options. */
    $optpage_blocks[] = array(
        'name' =>_("Highlighting"),
	'div'  => "div5",	
        'url'  => $base_uri.'src/options_highlight.php',
        'desc' =>_("Based upon given criteria, incoming messages can have different background colors in the message list.  This helps to easily distinguish who the messages are from, especially for mailing lists."),
        'js'   => false
    );

    /* Build a section for Folder Options. */
    $optpage_blocks[] = array(
        'name' => _("Folders"),
	'div'  => "div6",	
        'url'  => $base_uri.'src/options.php?optpage=' . SMOPT_PAGE_FOLDER,
        'desc' => _("These settings change the way your folders are displayed and manipulated."),
        'js'   => false
    );

    /* Build a section for Index Order Options. */
    $optpage_blocks[] = array(
        'name' => _("Indexes"),
	'div'  => "div7",	
        'url'  => $base_uri.'src/options_order.php',
        'desc' => _("The order of the message index can be rearranged and changed to contain the headers in any order you want."),
        'js'   => false
    );
    if ($use_pop3) { 
	$optpage_blocks[] = array(
    	     'name' => _("Fetch Mail"),
	     'div'  => "div8",	
    	     'url'  => $base_uri.'plugins/mail_fetch/options_fetch_mail.php',
    	     'desc' => _("This configures settings for downloading email from a pop3 mailbox to your account on this server."),
    	     'js'   => false
	);
    }	    

    $optpage_blocks[] = array(
         'name' => _("Filters"),
	 'div'  => "div9",	
         'url'  => $base_uri.'plugins/filters/options_filters.php',
         'desc' => _("Filtering enables messages with different criteria to be automatically filtered into different folders for easier organization."),
         'js'   => false
    );


    /*****************************************************/
    /* Let's sort Javascript Option Pages to the bottom. */
    /*****************************************************/
    $js_optpage_blocks = array();
    $reg_optpage_blocks = array();
    foreach ($optpage_blocks as $cur_optpage) {
        if (!$cur_optpage['js']) {
            $reg_optpage_blocks[] = $cur_optpage;
        } else if ($javascript_on == SMPREF_JS_ON) {
            $js_optpage_blocks[] = $cur_optpage;
        }
    }
    $optpage_blocks = array_merge($reg_optpage_blocks, $js_optpage_blocks);

    /********************************************/
    /* Now, print out each option page section. */
    /********************************************/
    echo '<tr>
	    <td width="100%" height="1%" align="center" valign="top">
		<table width="100%" class="optionnav" cellpadding="2" cellspacing="3" border="0">
		    <tr>';
    foreach ($optpage_blocks as $next_optpage) {
        print_optionpages_row($next_optpage);
    }
    echo '<td class="optionnav" align="center" width="9%" onClick="window.close();" valign="middle">
            '.codeHTML(_("Close")).'
        </td>';
		echo "</tr>
		</table>
	    </td>
	</tr>\n";
    echo '<script language="javascript" type="text/javascript" src="'.$base_uri.'js/new_window.js"></script>'."\n";
}

/**
 * This function prints out an option page row.
 */
function print_optionpages_row($leftopt) {
    global $optpage_name, $echotips;
    if ($optpage_name == $leftopt['name']) {
	$class="optionselected";
    } else {
	$class="optionnav";
    }
    echo '<td onMouseOver="showtip(event,\''.$leftopt['div'].'\')" onMouseOut="hidetip(\''.$leftopt['div'].'\')" 
	    class="'.$class.'" onClick="location.href=\''.$leftopt['url'].'\'" align="center" width="12%" valign="middle">
            ' . codeHTML($leftopt['name']) . '
	</td>';
    $echotips.= '<div id="'.$leftopt['div'].'" style="position:absolute;z-index:16;visibility:hidden;top:0px;left:0px;">    
	    <table class="optiontip" width="150" border=0 cellspacing=1 cellpadding=1>
		<tr>
		    <td aling="left" valign="top">
            		'. codeHTML($leftopt['desc']) .'
		    </td>
		</tr>
	    </table>
	</div>';
}

?>
