<?php

/********************************************************************
 *   Autori:             Marco Colombo (marco@openit.it)
 *                       Valentina Portolan (valentina@openit.it)
 *                       Giuliano Natali Diaolin (diaolin@openit.it)
 *  Copyright:           Open It S.r.l.
 *                       Viale Dante, 78
 *                       38057 Pergine Valsugana (TN) ITALY
 *                       Tel: +39 0461 534800 Fax: +39 0461 538443
 *********************************************************************/

/**
 * options.php
 *
 * Copyright (c) 1999-2003 The SquirrelMail Project Team
 * Licensed under the GNU GPL. For full terms see the file COPYING.
 *
 * Displays the options page. Pulls from proper user preference files
 * and config.php. Displays preferences as selected and other options.
 *
 * $Id: options.php,v 1.2 2004/02/25 15:02:08 marco Exp $
 */

/* Path for SquirrelMail required files. */
define('SM_PATH','../');

/* SquirrelMail required files. */
require_once(SM_PATH . 'include/validate.php');
require_once(SM_PATH . 'functions/global.php');
require_once(SM_PATH . 'functions/display_messages.php');
require_once(SM_PATH . 'functions/imap.php');
require_once(SM_PATH . 'functions/options.php');
require_once(SM_PATH . 'functions/strings.php');
require_once(SM_PATH . 'functions/html.php');
require_once(SM_PATH . 'functions/forms.php');

/*********************************/
/*** Build the resultant page. ***/
/*********************************/

function process_optionmode_submit($optpage, $optpage_data) {
    /* Initialize the maximum option refresh level. */
    $max_refresh = SMOPT_REFRESH_NONE;

    /* Save each option in each option group. */
    foreach ($optpage_data['options'] as $option_grp) {
        foreach ($option_grp['options'] as $option) {
            /* Remove Debug Mode Until Needed
            echo "name = '$option->name', "
               . "value = '$option->value', "
               . "new_value = '$option->new_value'\n";
            echo "<br>";
            */
            if ($option->changed()) {
                $option->save();
                $max_refresh = max($max_refresh, $option->refresh_level);
            }
        }
    }

    /* Return the max refresh level. */
    return ($max_refresh);
}

function process_optionmode_link($optpage) {
   /* There will be something here, later. */
}

/* ---------------------------- main ---------------------------- */

/* get the globals that we may need */
sqgetGlobalVar('key',       $key,           SQ_COOKIE);
sqgetGlobalVar('username',  $username,      SQ_SESSION);
sqgetGlobalVar('onetimepad',$onetimepad,    SQ_SESSION);
sqgetGlobalVar('delimiter', $delimiter,     SQ_SESSION);
sqgetGlobalVar('base_uri', $base_uri,     SQ_SESSION);

sqgetGlobalVar('optpage',     $optpage, 	SQ_POST);
sqgetGlobalVar('optmode',     $optmode, 	SQ_POST);
sqgetGlobalVar('optpage_data',$optpage_data, 	SQ_POST);
/* end of getting globals */

if (!$optpage) {
    sqgetGlobalVar('optpage',     $optpage,         SQ_GET);
}

sqgetGlobalVar('askinfo',     $askinfo,         SQ_GET);
sqgetGlobalVar('askinfo_ok',     $askinfo_ok,         SQ_POST);

/* Make sure we have an Option Page set. Default to main. */
if ( !isset($optpage) || $optpage == '' ) {
    $optpage = SMOPT_PAGE_MAIN;
} else {
    $optpage = strip_tags( $optpage );
}

/* Make sure we have an Option Mode set. Default to display. */
if (!isset($optmode)) {
    $optmode = SMOPT_MODE_DISPLAY;
}

/*
 * First, set the load information for each option page.   
 */

/* Initialize load information variables. */
$optpage_name = '';
$optpage_file = '';
$optpage_loader = '';
$echotips="";

/****************************************************************/

/* Set the load information for each page. */
switch ($optpage) {
    case SMOPT_PAGE_MAIN:
        $optpage_name     = _("Personal");
        $optpage_file     = SM_PATH . 'include/options/personal.php';
        $optpage_loader   = 'load_optpage_data_personal';
        $optpage_loadhook = 'optpage_loadhook_personal';
        break;
    case SMOPT_PAGE_PERSONAL:
        $optpage_name     = _("Personal");
        $optpage_file     = SM_PATH . 'include/options/personal.php';
        $optpage_loader   = 'load_optpage_data_personal';
        $optpage_loadhook = 'optpage_loadhook_personal';
        break;
    case SMOPT_PAGE_DISPLAY:
        $optpage_name   = _("General");
        $optpage_file   = SM_PATH . 'include/options/display.php';
        $optpage_loader = 'load_optpage_data_display';
        $optpage_loadhook = 'optpage_loadhook_display';
        break;
    case SMOPT_PAGE_HIGHLIGHT:
        $optpage_name   = _("Highlighting");
        $optpage_file   = SM_PATH . 'include/options/highlight.php';
        $optpage_loader = 'load_optpage_data_highlight';
        $optpage_loadhook = 'optpage_loadhook_highlight';
        break;
    case SMOPT_PAGE_FOLDER:
        $optpage_name   = _("Folders");
        $optpage_file   = SM_PATH . 'include/options/folder.php';
        $optpage_loader = 'load_optpage_data_folder';
        $optpage_loadhook = 'optpage_loadhook_folder';
        break;
    case SMOPT_PAGE_ORDER:
        $optpage_name = _("Indexes");
        $optpage_file = SM_PATH . 'include/options/order.php';
        $optpage_loader = 'load_optpage_data_order';
        $optpage_loadhook = 'optpage_loadhook_order';
        break;
    default: do_hook('optpage_set_loadinfo');
}

/**********************************************************/
/*** Second, load the option information for this page. ***/
/**********************************************************/

    /* Include the file for this optionpage. */
    require_once($optpage_file);

    /* Assemble the data for this option page. */
    $optpage_data = array();
    $optpage_data = $optpage_loader();
    do_hook($optpage_loadhook);
    $optpage_data['options'] =
        create_option_groups($optpage_data['grps'], $optpage_data['vals']);

/***********************************************************/
/*** Next, process anything that needs to be processed. ***/
/***********************************************************/

if ( isset( $optpage_data ) ) {
    switch ($optmode) {
        case SMOPT_MODE_SUBMIT:
            $max_refresh = process_optionmode_submit($optpage, $optpage_data);
            break;
        case SMOPT_MODE_LINK:
            $max_refresh = process_optionmode_link($optpage, $optpage_data);
            break;
    }
}

    /* Assemble the data for this option page. */
    $optpage_data = array();
    $optpage_data = $optpage_loader();
    do_hook($optpage_loadhook);
    $optpage_data['options'] =
        create_option_groups($optpage_data['grps'], $optpage_data['vals']);


$optpage_title = _("Options");
if (isset($optpage_name) && ($optpage_name != '')) {
    $optpage_title .= " - $optpage_name";
}

/***************************************************************/
/* Finally, display whatever page we are supposed to show now. */
/***************************************************************/

if ($optmode == SMOPT_MODE_SUBMIT && $askinfo_ok == 1) {
    header('Location: '.$base_uri.'src/webmail.php');
    exit;    
}

$js = '<script language="javascript">
    function OnSubmit() {
    ';
    if ($optpage_name == _("Personal")) {
	$js .= '
	    var a = document.f; 
	    if (CtrlNull(a.new_full_name.value)){
        	alert("'.jsspecialchars(_("Insert your full name!")).'");
        	a.new_full_name.focus();
        	return false;
    	    } else if (CtrlNull(a.new_email_address.value)) {
        	alert("'.jsspecialchars(_("Insert you email address!")).'");
        	a.new_email_address.focus();
        	return false;
    	    } else if (!CtrlNull(a.new_email_address.value) && !CtrlAT(a.new_email_address.value)) {
        	alert("'.jsspecialchars(_("Your email address is uncorrect!")).'");
        	a.new_email_address.focus();
        	return false;
	}';
    }	
$js .='	return true;	
    }
</script>';

    displayHtmlHeader($optpage_title,$js);
    echo '<script language="javascript" type="text/javascript" src="'.$base_uri.'js/new_window.js"></script>'."\n";
    echo '<script language="javascript" type="text/javascript" src="'.$base_uri.'js/ctrl_form.js"></script>';
    echo '<script language="javascript" type="text/javascript" src="'.$base_uri.'js/tips.js"></script>';
    echo '<body onload="self.focus();">';
    echo '<table class="border" width="100%" height="100%" cellpadding="0" cellspacing="0" border="0">';

if (!$askinfo) {
    ShowOptionBar(); 
} else {
    echo '<tr>
            <td class="error" align="center" height="1%" width="100%" valign="middle">
            '.codeHTML(_("You need to supply your full name and email adress first!")).'
            </td>
         </tr>';
}

/**********************/
/* corpo della pagina */
/**********************/
if ($optmode == SMOPT_MODE_SUBMIT) {
    echo '<tr>
	    <td class="optionselected" align="center" height="100%" width="100%" valign="middle">
	    '.codeHTML(_("Successfully Saved Options")).'
	    </td>
	 </tr>';
} else {
    echo '<tr>
	    <td align="left" width="100%" valign="top">
		<table width="100%" height="100%" cellpadding="0" valign="top" class="border" cellspacing="2" border="0">';
    echo addForm($base_uri.'src/options.php', 'POST', 'f');
    echo create_optpage_element($optpage) . create_optmode_element(SMOPT_MODE_SUBMIT);

    if ($askinfo) { echo addHidden('askinfo_ok', '1'); }

    /* Output the option groups for this page. */
    foreach ($optpage_data['options'] as $next_optgrp) {

        /* print the name for this option group. */
        if ($next_optgrp['name'] == '') { $next_optgrp['name']="&nbsp;"; }
	echo '<tr height="1%">
		<td align="left" width="100%" height="1%" valign="middle" colspan=2 class="optionlink">
		    '.codeHTML($next_optgrp['name']).'
		</td>	 
	    </tr>';   
 
        /* Print each option in this option group. */
        foreach ($next_optgrp['options'] as $option) {
            if ($option->type != SMOPT_TYPE_HIDDEN) {
		if ($option->name == "signature") { $valign="top"; } else { $valign="middle"; }
		echo '<tr height="1%">
			<td align="left" width="50%" valign="'.$valign.'" class="nav">
			    '.codeHTML($option->caption).':
			</td>
			<td align="left" width="50%" valign="'.$valign.'" class="nav">
			    '.$option->createHTMLWidget().'
			</td>
		    </tr>';
            } else {
                echo $option->createHTMLWidget();
            }
        }
    }

    /* Set the inside_hook_name and submit_name. */
    switch ($optpage) {
        case SMOPT_PAGE_MAIN:
            $inside_hook_name = 'options_personal_inside';
            $bottom_hook_name = 'options_personal_bottom';
            $submit_name = 'submit_personal';
            break;
        case SMOPT_PAGE_PERSONAL:
            $inside_hook_name = 'options_personal_inside';
            $bottom_hook_name = 'options_personal_bottom';
            $submit_name = 'submit_personal';
            break;
        case SMOPT_PAGE_DISPLAY:
            $inside_hook_name = 'options_display_inside';
            $bottom_hook_name = 'options_display_bottom';
            $submit_name = 'submit_display';
            break;
        case SMOPT_PAGE_HIGHLIGHT:
            $inside_hook_name = 'options_highlight_inside';
            $bottom_hook_name = 'options_highlight_bottom';
            $submit_name = 'submit_highlight';
            break;
        case SMOPT_PAGE_FOLDER:
            $inside_hook_name = 'options_folder_inside';
            $bottom_hook_name = 'options_folder_bottom';
            $submit_name = 'submit_folder';
            break;
        case SMOPT_PAGE_ORDER:
            $inside_hook_name = 'options_order_inside';
            $bottom_hook_name = 'options_order_bottom';
            $submit_name = 'submit_order';
            break;
        default:
            $inside_hook_name = '';
            $bottom_hook_name = '';
            $submit_name = 'submit';
    }

    /* If it is not empty, trigger the inside hook. */
    if ($inside_hook_name != '') {
        do_hook($inside_hook_name);    
    }

    /* Spit out a submit button. */
    echo '<tr height="1%">
	    <td colspan=2 width="100%" align="center" class="nav" valign="middle" height="1%">';
	echo addHidden($submit_name, _("Submit"));
	echo addButton(_("Submit"), '','','','f','OnSubmit()');
    echo '  </td>
	</tr>';
    echo '<tr>
                <td align="left" width="100%" class="nav" colspan=2>
                    &nbsp;
                </td>
            </tr>'."\n";
    echo '</form></table></td></tr>';

    /* If it is not empty, trigger the bottom hook. */
    if ($bottom_hook_name != '') {
        do_hook($bottom_hook_name);    
    }
}
    echo '</table>';
    echo $echotips;
    echo '</body></html>';   

?>
